/*
  OSynC example capture code for Processing by Alex Mesker http://www.x37v.com Oct 08
  Illustrates hows how to receive OSC messages from the OSynC-lite VST Plugin
  Requires oscP5 by andreas schlegel http://www.sojamo.de/oscP5 to be added to the sketch and a VST host running OSynC-lite
  OSynC-lite VST Plugin transmits basic host transport information to localhost on port 10101 
*/

import processing.net.*;
import oscP5.*;
import netP5.*;

PFont font;
OscP5 oscP5;
NetAddress fromOSynC;
int playValue, barcountValue, barValue, beatValue, fractionValue;
float msticksValue;
String playStatus;

void setup() {
  size(400,400);
  frameRate(32);
  font = loadFont("OCRAStd-48.vlw");
  textAlign(CENTER);
  color(255);

  /* start oscP5, listening for incoming messages at port 10101 */
  oscP5 = new OscP5(this, 10101);
  
  fromOSynC = new NetAddress("127.0.0.1", 10101);
}


void draw() {
  background(0);
  textFont(font, 48);
  text(nf(barValue, 2) + ":" + nf(beatValue, 2) + ":" + nf(fractionValue, 2), width/2, height/2);
    if (playValue == 1) {
      playStatus = "PLAYING";
    } else {
      playStatus = "STOPPED";
    }
  textFont(font, 24);
  text(playStatus, width/2, height/2 + 50);
  text("Bar: " + barcountValue, width/2, height/2 + 75);
  textFont(font, 16);
  text(nf(msticksValue, 0, 2) + "ms between fractions", width/2, height/2 + 100);
}


void oscEvent(OscMessage theOscMessage) {
  /* parse the OSC message based on prefix and extract the values from the osc message arguments.  */
  
  if(theOscMessage.checkAddrPattern("/play")==true) {
    if(theOscMessage.checkTypetag("i")) { /* /play returns an int.  1 = playing, 0 = stopped */
      playValue = theOscMessage.get(0).intValue();
      return;
    }  
  }  else if(theOscMessage.checkAddrPattern("/barcount")==true) {
    if(theOscMessage.checkTypetag("i")) { /* /barcount returns the total bar number from the VST host */
      barcountValue = theOscMessage.get(0).intValue();  
      return;
    }  
  } else if(theOscMessage.checkAddrPattern("/bar")==true) {
    if(theOscMessage.checkTypetag("i")) { /* /bar returns the total bar number % 4 from the VST host.  Useful for keeping track of position within a phrase */
      barValue = theOscMessage.get(0).intValue();  
      return;
    }  
  } else if(theOscMessage.checkAddrPattern("/beat")==true) {
    if(theOscMessage.checkTypetag("i")) { /* /beat returns position within a bar */
      beatValue = theOscMessage.get(0).intValue();  
      return;
    }
  } else if(theOscMessage.checkAddrPattern("/fraction")==true) {
    if(theOscMessage.checkTypetag("i")) { /* /fraction returns either 16th or 32nd fractions of the bar */
      fractionValue = theOscMessage.get(0).intValue();  
      return;
    }  
  } else if(theOscMessage.checkAddrPattern("/msticks")==true) {
    if(theOscMessage.checkTypetag("f")) { /* /msticks returns a float millisecond length between fraction increments */
      msticksValue = theOscMessage.get(0).floatValue();
      return;
    }  
  }
  
  println("### received an osc message. with address pattern "+theOscMessage.addrPattern());
}
